@@ -261,8 +261,13 @@ class ParallelAtoms(ase.Atoms):
     
     def get_temperature(self):
         """Get the temperature. in Kelvin"""
-        ekin = self.get_kinetic_energy() / self.get_global_number_of_atoms()
-        return ekin / (1.5 * ase.units.kB)
+        dof = 3 * self.get_global_number_of_atoms()
+        removed_dof = 0
+        for c in self.constraints:
+            removed_dof += c.get_removed_dof(self)
+        removed_dof = self.comm.sum(int(removed_dof))
+        ekin_per_dof = self.get_kinetic_energy() / (dof - removed_dof)
+        return ekin_per_dof / (0.5 * ase.units.kB)
 
     def get_ghost_positions(self):
         return self.ghosts['positions'].copy()
@@ -394,6 +399,9 @@ class ParallelAtoms(ase.Atoms):
         atom is deleted if it is specified in at least one task.
         Duplicates are allowed.
         """
+        for c in self.constraints:
+            # Constraint must have data stored on atoms.
+            c.start_asap_dynamics(self)
         indices = []
         if isinstance(local_indices, int):
             local_indices = [local_indices]
@@ -464,7 +472,8 @@ class ParallelAtoms(ase.Atoms):
         for constraint in self.constraints:
             if not getattr(constraint, 'asap_ready', False):
                 raise ValueError("Deleting atoms has corrupted contraint "+str(constraint))
-            
+        for c in self.constraints:
+            c.end_asap_dynamics(self)
 
     # It is not possible to delete atoms with __delitem__ as that cannot sync correctly.
     def __delitem__(self, *args, **kwargs):
