@@ -15,9 +15,6 @@ class ASE_Langevin_v3_or_4(Langevin_ASE, ParallelMolDynMixin, object):
     def __init__(self, atoms, timestep, temperature=None, friction=None, fixcm=True,
                      *, temperature_K=None, **kwargs):
         ParallelMolDynMixin.__init__(self, "Langevin", atoms)
-        for constraint in atoms.constraints:
-            if hasattr(constraint, "prepare_for_asap"):
-                constraint.prepare_for_asap(atoms)
         if Langevin_ASE._lgv_version == 3:
             if temperature_K is not None:
                 assert temperature is None
@@ -26,11 +23,16 @@ class ASE_Langevin_v3_or_4(Langevin_ASE, ParallelMolDynMixin, object):
                                   friction=friction, fixcm=fixcm,
                                   communicator=None, **kwargs)
         else:
-            assert Langevin_ASE._lgv_version == 4
+            assert Langevin_ASE._lgv_version in (4, 5)
             Langevin_ASE.__init__(self, atoms, timestep, temperature=temperature,
                                   friction=friction, temperature_K=temperature_K,
                                   fixcm=fixcm, communicator=None, **kwargs)
 
+    def run(self, steps):
+        self.before_run()
+        super().run(steps)
+        self.after_run()
+
     def _get_com_velocity(self, velocity=None):
         """Return the center of mass velocity."""
         if velocity is None:
@@ -53,8 +55,11 @@ class ASE_Langevin_v3_or_4(Langevin_ASE, ParallelMolDynMixin, object):
     c4 = property(lambda s: s.get("c4"), lambda s, x: s.set("c4", x))
     c5 = property(lambda s: s.get("c5"), lambda s, x: s.set("c5", x))
     v = property(lambda s: s.get("v"), lambda s, x: s.set("v", x))
+    # Depending on the ASE version, either xi and eta will exist, or rnd_vel and rnd_pos
     xi = property(lambda s: s.get("xi"), lambda s, x: s.set("xi", x))
     eta = property(lambda s: s.get("eta"), lambda s, x: s.set("eta", x))
+    rnd_pos = property(lambda s: s.get("rnd_pos"), lambda s, x: s.set("rnd_pos", x))
+    rnd_vel = property(lambda s: s.get("rnd_vel"), lambda s, x: s.set("rnd_vel", x))
 
 class Langevin_Fast(MolecularDynamics, ParallelMolDynMixin, object):
     def __init__(self, atoms, timestep, temperature=None, friction=None, fixcm=True,
@@ -65,18 +70,20 @@ class Langevin_Fast(MolecularDynamics, ParallelMolDynMixin, object):
             if temperature_K is not None:
                 temperature = temperature_K * ase.units.kB
         else:
-            assert Langevin_ASE._lgv_version == 4
+            assert Langevin_ASE._lgv_version >= 4
             temperature = ase.units.kB * self._process_temperature(temperature, temperature_K, 'eV')
         ParallelMolDynMixin.__init__(self, "Langevin", atoms)
         self._uselocaldata = False # Need to store on atoms for serial simul too.
         self.calculator = atoms.get_calculator()
         if not atoms.has('momenta'):
             atoms.set_momenta(np.zeros((len(atoms), 3), float))
-        if atoms.constraints:
-            assert len(atoms.constraints) == 1
+        self.atoms_have_constraints = len(atoms.constraints)
+        if self.atoms_have_constraints:
+            if len(atoms.constraints) != 1:
+                raise RuntimeError("ASAP Langevin dynamics only support a single constraint.")
             constraint = atoms.constraints[0]
-            assert isinstance(constraint, asap3.constraints.FixAtoms)
-            constraint.prepare_for_asap(atoms)
+            if not isinstance(constraint, asap3.constraints.FixAtoms):
+                raise RuntimeError("ASAP Langevin dynamics only support the ASAP FixAtoms constraint.")
             # Make all constants arrays by making friction an array
             friction = friction * np.ones(len(atoms))
             fixcm = False   # Unneccesary (and incompatible) when FixAtoms constraint used.
@@ -169,7 +176,13 @@ class Langevin_Fast(MolecularDynamics, ParallelMolDynMixin, object):
 
     def run(self, steps):
         assert(self.calculator is self.atoms.get_calculator())
+        if self.atoms.constraints or self.atoms_have_constraints:
+            # Constraints may have been added, removed or changed
+            self.updatevars()
+        self.before_run()
         self.asap_md.run(steps, self.observers, self)
+        self.after_run()
+        self.atoms_have_constraints = len(self.atoms.constraints)
 
     def get_random(self, gaussian):
         return self.asap_md.get_random(gaussian)
@@ -185,16 +198,17 @@ class Langevin_Fast(MolecularDynamics, ParallelMolDynMixin, object):
     pmcor = property(lambda s: s.get("pmcor"), lambda s, x: s.set("pmcor", x))
     cnst = property(lambda s: s.get("cnst"), lambda s, x: s.set("cnst", x))
     masses = property(lambda s: s.get("masses"), lambda s, x: s.set("masses", x))
-
+    frict = property(lambda s: s.get("frict"), lambda s, x: s.set("frict", x))
 
 def Langevin(atoms, *args, **kwargs):
     if (isinstance(atoms, ase.Atoms)
         and asap3.constraints.check_asap_constraints(atoms)
+        and not kwargs.pop('forceASE', False)
        ):
         # Nothing prevents Asap optimization
         sys.stderr.write("Using Asap-optimized C++-Langevin algorithm\n")
         return Langevin_Fast(atoms, *args, **kwargs)
-    elif Langevin_ASE._lgv_version in (3, 4):
+    elif Langevin_ASE._lgv_version in (3, 4, 5):
         # ASE Langevin is version 3
         sys.stderr.write("Using ASE-based Langevin algorithm\n")
         return ASE_Langevin_v3_or_4(atoms, *args, **kwargs)
